/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef FILTER_INTERSECTOR_HH_
# define FILTER_INTERSECTOR_HH_

# include <wipal/tool/options.hh>

namespace wpl
{

  namespace filter
  {

    /**
     * Build an intersector from iterators and pass it as argument
     * to a functor.
     *
     * @c options must contain at least the following items:
     * mapping, endianness, and ignore_noisy_prism.
     *
     * @c phy_end indicates the PHY headers' endianness.
     */
    template <class UniqueId,
	      class HeaderType,
	      template <class, class, class> class Intersector,
	      class I1, class I2, class F>
    void
    provide_intersector(const I1& first1, const I1& last1,
			const I2& first2, const I2& last2,
			opt::list&	options,
			F&		func);

  } // End of namespace wpl::filter.

  } // End of namespace wpl.

# include "intersector.hxx"

#endif // ! FILTER_INTERSECTOR_HH_
