/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef FILTER_INTERSECTOR_HXX_
# define FILTER_INTERSECTOR_HXX_

# include "intersector.hh"

# include <wipal/filter/non_noisy_prism.hh>
# include <wipal/filter/uniquely_identifiable.hh>

namespace wpl
{

  namespace filter
  {

    template <class U, class HT, template <class, class, class> class Int,
	      class I1, class I2, class F>
    void
    provide_intersector(const I1& first1, const I1& last1,
			const I2& first2, const I2& last2,
			opt::list&	options,
			F&		func)
    {
      typedef tool::end::endianness	endianness;

      const unsigned&	   prec = options["precision"].get<unsigned>();
      const endianness&	   end  = options["endianness"].get<endianness>();
      wifi::addr_mapping&  map  = options["mapping"].get<wifi::addr_mapping>();
      const bool	   pcapts = not options["phy_tstamps"].get<bool>();

      if (options["ignore_noisy_prism"].get<bool>())
	{
	  typedef non_noisy_prism<I1>			nnp1;
	  typedef non_noisy_prism<I2>			nnp2;
	  typedef typename nnp1::const_iterator		nnp1_i;
	  typedef typename nnp2::const_iterator		nnp2_i;
	  typedef uniquely_identifiable<U, nnp1_i, HT>	ui1;
	  typedef uniquely_identifiable<U, nnp2_i, HT>	ui2;
	  typedef typename ui1::const_iterator		ui1_i;
	  typedef typename ui2::const_iterator		ui2_i;
	  typedef Int<ui1_i, ui2_i, tool::bottom>	intersector;

	  nnp1		n1 (first1, last1);
	  nnp2		n2 (first2, last2);
	  ui1		u1 (n1.begin(), n1.end(), map, end, pcapts);
	  ui2		u2 (n2.begin(), n2.end(), map, end, pcapts);
	  intersector	i (u1.begin(), u1.end(), u2.begin(), u2.end(), prec);

	  func(i);
	}
      else
	{
	  typedef uniquely_identifiable<U, I1, HT>	ui1;
	  typedef uniquely_identifiable<U, I2, HT>	ui2;
	  typedef typename ui1::const_iterator		ui1_i;
	  typedef typename ui2::const_iterator		ui2_i;
	  typedef Int<ui1_i, ui2_i, tool::bottom>	intersector;

	  ui1		u1 (first1, last1, map, end, pcapts);
	  ui2		u2 (first2, last2, map, end, pcapts);
	  intersector	i (u1.begin(), u1.end(), u2.begin(), u2.end(), prec);

	  func(i);
	}
    }

  } // End of namespace wpl::filter.

} // End of namespace wpl.

#endif // ! FILTER_INTERSECTOR_HXX_
