/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef FILTER_MERGE_HH_
# define FILTER_MERGE_HH_

# include <wipal/pcap/pkt.hh>
# include <wipal/tool/endianness.hh>

namespace wpl
{

  namespace filter
  {

    namespace internals
    {

      template <class HT, class S1>
      struct equal
      {
	equal(const pkt::packet<S1>&		lhs,
	      const unsigned&			precision,
	      const tool::end::endianness&	phy_end,
	      const bool&			force_pcap);

	template <class S2>
	bool
	operator == (const pkt::packet<S2>& rhs) const;

      private:
	const pkt::packet<S1>&		lhs_;
	const unsigned&			precision_;
	const tool::end::endianness&	phy_end_;
	const bool&			force_pcap_;
      };

      template <class HT, class S1>
      equal<HT, S1>
      make_equal(const pkt::packet<S1>&		lhs,
		 const unsigned&		precision,
		 const tool::end::endianness&	phy_end,
		 const bool&			force_pcap);

      template <class HT, class S1, class S2>
      bool
      operator == (const pkt::packet<S2>&	lhs,
		   const equal<HT, S1>&		rhs);

    } // End of namespace internals.

    /**
     * Build an merge filter from iterators and pass it as argument
     * to a functor.
     *
     * @c options must contain at least the following items:
     * mapping, endianness, ignore_noisy_prism, and ref_blacklist.
     *
     * @c phy_end indicates the PHY headers' endianness.
     */
    template <class UniqueId,
	      class HeaderType,
	      template <class, class, class> class Intersector,
	      template <class, class, class, class> class Merger,
	      class I1, class I2, class F>
    void
    provide_merge(const I1& first1, const I1&	last1,
		  const I2& first2, const I2&	last2,
		  opt::list&		options,
		  F&			func);

  } // End of namespace wpl::filter.

} // End of namespace wpl.

# include "merge.hxx"

#endif // ! FILTER_MERGE_HH_
