/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef FILTER_TIMETRACKER_HH_
# define FILTER_TIMETRACKER_HH_

# include <wipal/tool/microseconds_stamp.hh>
# include <wipal/tool/valued_iterator.hh>
# include <wipal/phy/timetracker.hh>
# include <wipal/tool/iterable.hh>

# include "timetracker_fwd.hh"

namespace wpl
{

  /*--------------.
  | Virtual types |
  `--------------*/

  namespace tool
  {

    template <class I, class HT, class B1, class B2>
    struct types< filter::internals::timetracker_iterator<I, HT, B1, B2> >
    {
      typedef filter::timetracker<I, HT, B1>			iterable_type;
      typedef microseconds_stamp<typename I::value_type>	value_type;
    };

    template <class I, class HT, class B>
    struct types< filter::timetracker<I, HT, B> >
    {
      typedef
      filter::internals::timetracker_iterator<I, HT, B, bottom>
      iterator;
    };

  } // End of namespace tool.

  /*-------------.
  | Declarations |
  `-------------*/

  namespace filter
  {

    namespace internals
    {

      template <class InputIterator,
		class HeaderType,
		class B,
		class Bottom = tool::bottom>
      struct timetracker_iterator:
	WP_INHERIT(public tool::valued_iterator,
		   timetracker_iterator<InputIterator, HeaderType, B, Bottom>)
      {
	/// @name Static hierarchies related types
	//@{
	typedef WP_GET_EXACT(Bottom, timetracker_iterator<InputIterator,
							  HeaderType, B,
							  Bottom>)
							exact_type;
	typedef tool::valued_iterator<exact_type>	super_type;
	typedef WP_TYPE(value_type, exact_type)		value_type;
	typedef WP_TYPE(iterable_type, exact_type)	iterable_type;
	//@}

	/// tool::iterable compatible constructor.
	timetracker_iterator(const iterable_type& i, bool end);

	/// @name Abstract method implementations
	//@{
	bool	equal(const timetracker_iterator& rhs) const;
	void	increment();
	//@}

      private:
	const iterable_type*	iterable_;
	InputIterator		next_;
	phy::timetracker<>	tracker_;
      };

    } // End of namespace internals.

    template <class InputIterator,
	      class HeaderType,
	      class Bottom = tool::bottom>
    struct timetracker:
      WP_INHERIT(public tool::iterable, timetracker<InputIterator,
		 HeaderType,
		 Bottom>)
    {
      /// Construct from two iterators.
      timetracker(const InputIterator&	first,
		  const InputIterator&	last,
		  tool::end::endianness	phy_end = tool::end::guess,
		  bool			pcap_tstamps = false);

    private:
      InputIterator		first_;
      InputIterator		last_;
      tool::end::endianness	phy_end_;
      bool			pcap_tstamps_;

      friend
	class internals::timetracker_iterator<InputIterator, HeaderType,
					      Bottom, tool::bottom>;
    };

  } // End of namespace wpl::filter.

} // End of namespace wpl.

# include "timetracker.hxx"

#endif // ! FILTER_TIMETRACKER_HH_
