/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef FILTER_TIMETRACKER_HXX_
# define FILTER_TIMETRACKER_HXX_

# include "timetracker.hh"

namespace wpl
{

  namespace filter
  {

    namespace internals
    {

      template <class I, class HT, class B1, class B2>
      timetracker_iterator<I, HT, B1, B2>::
      timetracker_iterator(const iterable_type& i, bool end):
	iterable_ (&i),
	next_ (end ? i.last_ : i.first_),
	tracker_ (i.pcap_tstamps_)
      {
	if (not end)
	  increment();
      }

      template <class I, class HT, class B1, class B2>
      bool
      timetracker_iterator<I, HT, B1, B2>::
      equal(const timetracker_iterator& rhs) const
      {
	if (not rhs.value())
	  return not this->value();

	const bool r (next_ == rhs.next_);

	assert(not r or
	       tracker_.microseconds() == rhs.tracker_.microseconds());

	return r;
      }

      template <class I, class HT, class B1, class B2>
      void
      timetracker_iterator<I, HT, B1, B2>::increment()
      {
	if (next_ == iterable_->last_)
	  {
	    this->value() = boost::none_t ();
	    return;
	  }

	tracker_.tick<HT>(*next_, iterable_->phy_end_);
	this->value() = value_type (tracker_.microseconds(), *next_);
	++next_;
      }

    } // End of namespace internals.

    template <class I, class HT, class B>
    timetracker<I, HT, B>::
    timetracker(const I&		first,
			 const I&		last,
			 tool::end::endianness	phy_end,
			 bool			pcap_tstamps):
      first_ (first),
      last_ (last),
      phy_end_ (phy_end),
      pcap_tstamps_ (pcap_tstamps)
    {
    }

  } // End of namespace wifi::frame::filter.

} // End of namespace wifi::frame.

#endif // ! FILTER_TIMETRACKER_HXX_
