/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef PCAP_BASE_HH_
# define PCAP_BASE_HH_

extern "C"
{
# include <inttypes.h>
# include <time.h>
}

namespace wpl
{

  namespace pcap
  {

    namespace internals
    {
      enum
	{
	  /// Current PCAP major version.
	  version_major	= 2,

	  /// Current PCAP minor version.
	  version_minor	= 4,

	  /// Maximum packet size w.r.t. the PCAP file format.
	  max_packet_size	= 65535
	};

      /// PCAP file format internal header.
      struct file_header
      {
	uint32_t	magic;		///< File magic.
	uint16_t	version_major;	///< PCAP format version, major.
	uint16_t	version_minor;	///< PCAP format version, minor.
	int32_t		thiszone;	///< Time offset.
	int32_t		sigfigs;	///< Timestamp accuracy.
	int32_t		snaplen;	///< Max. number of captured bytes.
	int32_t		linktype;	///< PCAP link type.
      } __attribute__ ((__packed__));

      /// PCAP file format representation of a packet header.
      struct file_frame_header
      {
	struct pcap_timeval
	{
	  uint32_t	tv_sec;
	  uint32_t	tv_usec;
	} __attribute__ ((__packed__));

	pcap_timeval	ts;	///< Timestamp.
	uint32_t	caplen;	///< Number of captured bytes.
	uint32_t	len;	///< %Packet len.
      } __attribute__ ((__packed__));

    } // End of namespace wpl::pcap::internals.

  } // End of namespace wpl::pcap.

} // End of namespace wpl.

#endif // ! PCAP_BASE_HH_
