/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef PCAP_FIND_HXX_
# define PCAP_FIND_HXX_

# include "find.hh"

# include <cassert>
# include <algorithm>

namespace wpl
{

  namespace pkt
  {

    inline
    match::match(const vector_of_bytes& bytes, size_t first, size_t last):
      bytes_ (bytes),
      first_ (first),
      last_ (last)
    {
      assert(first_ < last_);
      assert(bytes_.size() <= last_ - first_);
    }

    template <class S>
    bool
    match::operator == (const packet<S>& rhs) const
    {
      const size_t	caplen = rhs.meta().caplen;

      if (first_ > caplen)
	return false;

      const uint8_t* const  bytes = static_cast<const uint8_t*> (rhs.bytes());
      const uint8_t* const  first = bytes + first_;
      const uint8_t* const  last = bytes + std::min(caplen, last_);

      return std::search(first, last, bytes_.begin(), bytes_.end()) != last;
    }

    template <class S>
    bool
    operator == (const packet<S>& rhs, const match& lhs)
    {
      return lhs == rhs;
    }


  } // End of namespace wpl::pkt.

} // End of namespace wpl.

#endif // ! PCAP_FIND_HH_
