/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef PCAP_LIST_HH_
# define PCAP_LIST_HH_

# include <cstddef>
# include <string>
# include <utility>
# include <vector>

# include <wipal/tool/list_of_iterables.hh>
# include <wipal/pcap/stream.hh>

namespace wpl
{

  namespace pcap
  {

    template <class Stream = file<>, class Bottom = tool::bottom>
    struct list:
      public tool::list_of_iterables<Stream, WP_GET_EXACT(Bottom,
							  list<Stream, Bottom>)>
    {
      typedef WP_GET_EXACT(Bottom, list<Stream, Bottom>)	exact_type;
      typedef tool::list_of_iterables<Stream, exact_type>	super_type;

      enum
	{
	  trace_name_sep	= ':',
	  addr_sep	= '=',
	  escape_char	= '\\'
	};

      list(const std::string& stream_names = "");
      list(const list& other);

      list&		operator = (const list& rhs);

      std::ostream&	print(std::ostream&) const;

      pkt::type		type() const;
      void		expect(typename pkt::type t) const;
      const opt::list&	meta() const;

    protected:
      static
      std::pair<std::string, size_t>
      parse_word(const std::string&	stream_names,
		 const size_t		start,
		 const std::set<char>&	stop_seps);

      static
      std::pair<std::vector<std::string>, std::string>
      parse(const std::string& stream_names);

      void
      update_meta(list* p);

    private:
      opt::list		meta_;
    };

    template <class Stream, class Bottom>
    std::ostream&
    operator << (std::ostream& os, const list<Stream, Bottom>& l);

  } // End of namespace wpl::pcap.

  namespace tool
  {

    template <class S, class B>
    struct types< pcap::list<S, B> >:
      public types< list_of_iterables<S, WP_GET_EXACT(B, pcap::list<S, B>)> >
    {
    };

  } // End of namespace wpl::tool.

} // End of namespace wpl.

# include "list.hxx"

#endif // ! PCAP_LIST_HH_
