/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef PCAP_OSTREAM_HH_
# define PCAP_OSTREAM_HH_

extern "C"
{
# include <inttypes.h>
}
# include <string>
# include <vector>
# include <fstream>
# include <cstdlib>

# include <boost/shared_ptr.hpp>

# include <wipal/pcap/pkt.hh>
# include <wipal/pcap/stream.hh>
# include <wipal/pcap/list.hh>

namespace wpl
{

  namespace pcap
  {

    template <class B>
    struct stream;

    /**
     * Enable frame/packet dumping to PCAP files.
     *
     * A %pcap ostream allows you to dump packets to a file.
     */
    struct ostream
    {
      ostream(const std::string&	filename,
	      const pkt::type		type,
	      const int32_t		zone = 0,
	      const int32_t		snaplen = 0);

      template <class B>
      ostream(const std::string& filename, const stream<B>& stream);

      template <class S>
      ostream(const std::string& filename, const list<S>& list);

      /// Dump a packet given its meta-information and bytes.
      ostream&
      write(const pkt::metadata& meta, const void* bytes);

      /// Dump a @link pkt::packet network packet @endlink.
      template <class S>
      ostream&
      operator << (const pkt::packet<S>&);

      /// Same as operator << (for use with functors).
      template <class S>
      ostream&
      operator () (const pkt::packet<S>&);

      /// Retrieve a pcap::indexed_file object to the dumped file.
      pcap::indexed_file<>
      file() const;


    private:
      void		setup(const pkt::type	type,
			      const int32_t	zone,
			      const int32_t	snaplen);

      std::vector<std::streampos>	marks_;

      std::string			filename_;
      unsigned				size_;
      boost::shared_ptr<std::ofstream>	output_;
    };

  } // End of namespace wpl::pcap.

} // End of namespace wpl.

# include "ostream.hxx"

#endif // ! PCAP_STREAM_HH_
