/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef PHY_EMPTY_HXX_
# define PHY_EMPTY_HXX_

# include <wipal/phy/empty.hh>
# include <wipal/tool/timeval.hh>

namespace wpl
{

  namespace phy
  {

    /*----------------.
    | phy::empty_time |
    `----------------*/

    template <class B>
    typename empty_time<B>::impl_type
    empty_time<B>::get_impl() const
    {
      return impl_type ();
    }

    template <class B>
    typename empty_time<B>::exact_type&
    empty_time<B>::increment(tool::microseconds)
    {
      return this->exact();
    }


    /*------------------.
    | phy::empty_header |
    `------------------*/

    template <class B>
    size_t
    empty_header<B>::len_impl(size_t, bool) const
    {
      return 0;
    }

    template <class B>
    typename empty_header<B>::time_type
    empty_header<B>::time_get_impl(bool) const
    {
      return time_type ();
    }

    template <class B>
    void
    empty_header<B>::time_set_impl(const time_type&, bool)
    {
    }

    template <class B>
    template <class S1, class S2>
    bool
    empty_header<B>::eq_time(const pkt::packet<S1>&		lhs,
			     const pkt::packet<S2>&		rhs,
			     const unsigned			prec,
			     const tool::end::endianness	)
    {
      return tool::eq_timeval(lhs.meta().ts, rhs.meta().ts, prec);
    }

  } // End of namespace wpl::phy.

} // End of namespace wpl.

#endif // ! PHY_EMPTY_HXX_
