/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef RTAP_HEADER_HH_
# define RTAP_HEADER_HH_

# include <wipal/phy/phy.hh>
# include <wipal/tool/microseconds.hh>

namespace wpl
{

  // Forward declarations.
  namespace rtap
  {
    struct	header;
  }

  // Virtual types.
  namespace tool
  {

    template <>
    struct types<rtap::header>
    {
      typedef phy::uint64_time<>	time_type;
    };

  } // End of namespace wpl::tool.

  /// Radiotap %header related compounds.
  namespace rtap
  {

    /// Radiotap %header format.
    struct header: public phy::header<rtap::header>
    {
      typedef rtap::header			exact_type;
      typedef WP_TYPE_(time_type, exact_type)	time_type;

      size_t	len_impl(size_t, bool) const;
      time_type	time_get_impl(bool) const;
      void	time_set_impl(const time_type&, bool);

      template <class S1, class S2>
      static bool
      eq_time(const pkt::packet<S1>&		lhs,
	      const pkt::packet<S2>&		rhs,
	      const unsigned			prec,
	      const tool::end::endianness	phy_end);

      /**
       * Get the Radiotap %header's first field address (or 0 if not available).
       *
       * Const version.
       *
       * @warning This function does not check the radiotap header's version.
       */
      const void*
      first_field(size_t caplen) const;

      /**
       * Get the Radiotap %header's first field address (or 0 if not available).
       *
       * Non-const version.
       *
       * @warning This function does not check the radiotap header's version.
       */
      void*
      first_field(size_t caplen);

      /// Get the Radiotap header TSFT field, or throw @c std::invalid_argument.
      uint64_t
      tsft_get(size_t caplen) const;


      enum field
	{
	  tsft			= 0x00000001,
	  flags			= 0x00000002,
	  rate			= 0x00000004,
	  channel		= 0x00000008,
	  fhss			= 0x00000010,
	  dbm_antsignal		= 0x00000020,
	  dbm_antnoise		= 0x00000040,
	  lock_quality		= 0x00000080,
	  tx_attenuation	= 0x00000100,
	  db_tx_attenuation	= 0x00000200,
	  dbm_tx_power		= 0x00000400,
	  antenna		= 0x00000800,
	  db_antsignal		= 0x00001000,
	  db_antnoise		= 0x00002000,
	  ext			= 0x10000000,
	};

      uint8_t	it_version;	///< Must be zero.
      uint8_t	it_pad;		///< Padding; undefined.
      uint16_t	it_len;		///< Header's entire length.
      uint32_t	it_present;	///< Available fields.
    } __attribute__((__packed__));

  } // End of namespace wpl::rtap.

} // End of namespace wpl.

# include "rtap_header.hxx"

#endif // ! RTAP_HEADER_HH_
