/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef PROTO_EN2_HH_
# define PROTO_EN2_HH_

extern "C"
{
# include <inttypes.h>
}

# include <wipal/tool/addr.hh>

namespace wpl
{

  /// Ethernet II related compounds.
  namespace en2
  {

    /// Ethernet address.
    typedef tool::addr<uint8_t, 6, true, ':'>	addr;

    // Ethernet II EtherTypes.
    namespace type
    {

      // Ethernet II EtherType.
      enum ethertype
	{
	  IP	= 0x0800,
	  ARP	= 0x0806,
	  IPv6	= 0x86dd,
	  AARP	= 0x80f3
	};

    } // End of namespace en2::type.

    // Ethernet II frame.
    struct header
    {
      // Frame addresses index.
      enum addr_index
	{
	  da		= 0,	//< Destination address.
	  sa		= 1,	//< Source address.
	};

      addr		addrs[2];	//< Frame addresses.
      uint16_t		ethertype;	//< EtherType.
      uint8_t		data[];

    } __attribute__((__packed__));

  } // End of namespace wpl::en2.

} // End of namespace wpl.

#endif // ! PROTO_EN2_HH_
