/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef PROTO_IP4_HH_
# define PROTO_IP4_HH_

extern "C"
{
# include <inttypes.h>
}

# include <wipal/tool/addr.hh>

namespace wpl
{

  /// IPv4 related compounds.
  namespace ip4
  {

    /// IPv4 address.
    typedef tool::addr<uint8_t, 4, false, '.'>	addr;

    /// IPv4 protocols.
    namespace proto
    {

      /// IPv4 protocol.
      enum proto
	{
	  ICMP	= 1,
	  TCP	= 6,
	  UDP	= 17,
	  ICMPv6	= 58
	};

    } // End of namespace ip4::proto.

    /// IPv4 header.
    struct header
    {
      /// IPv4 addresses index.
      enum addr_index
	{
	  sa	= 0,	///< Source address.
	  da	= 1,	///< Destination address.
	};

      union
      {
	// Use a bitmask to retrieve the 4 first or last bits.
	uint8_t	version;
	uint8_t	headerlength;
      };

      uint8_t	dsfield;
      uint16_t	iplen;
      uint16_t	identifier;

      union
      {
	// Use a bitmask to retrieve the 3 first or 13 last bits.
	uint16_t	flags;
	uint16_t	fragoffset;
      };

      uint8_t	ttl;
      uint8_t	ipproto;
      uint16_t	checksum;
      addr	addrs[2];

    } __attribute__((__packed__));

  } // End of namespace wpl::ip4.

} // End of namespace wpl.

#endif // ! PROTO_IP4_HH_
