/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef TOOL_GMP_64_WRAPPER_HXX_
# define TOOL_GMP_64_WRAPPER_HXX_

# include <stdexcept>

# include <wipal/tool/gmp_64_wrapper.hh>

namespace wpl
{

  namespace tool
  {

    inline
    mpz_class
    mpz_of_uint64(uint64_t u)
    {
      const mpz_class	u_hi = uint32_t (u >> 32);
      const mpz_class	u_lo = uint32_t (u);

      return (u_hi << 32) | u_lo;
    }

    inline
    uint64_t
    uint64_of_mpz(const mpz_class u)
    {
      static const mpz_class	mask = uint32_t (0xFFFFFFFF);

      switch (sgn(u))
	{
	case -1:
	  {
	    const mpz_class	t = -u;
	    const mpz_class	l = t & mask;
	    const mpz_class	h = t >> 32;
	    const uint64_t	v = (uint64_t (h.get_ui()) << 32) | l.get_ui();

	    return -v;
	  }

	case 0:
	  return 0;

	case 1:
	  {
	    const mpz_class	l = u & mask;
	    const mpz_class	h = u >> 32;

	    return (uint64_t (h.get_ui()) << 32) | l.get_ui();
	  }

	default:
	  throw std::logic_error ("Internal error, please fill a bug report");
	}
    }

  } // End of namespace wpl::tool.

} // End of namespace wpl.

#endif // TOOL_GMP_64_WRAPPER_HXX_
