/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#ifndef WIPAL_TOOL_HEXDUMP_HH_
# define WIPAL_TOOL_HEXDUMP_HH_

extern "C"
{
# include <inttypes.h>
}

# include <ostream>
# include <vector>

namespace wpl
{

  namespace tool
  {

    namespace fmt
    {
      struct raw
      {
	std::ostream&	begin(std::ostream& os) const;
	std::ostream&	end(std::ostream& os) const;

	std::ostream&	bold_begin(std::ostream& os) const;
	std::ostream&	bold_end(std::ostream& os) const;

	std::ostream&	encode(std::ostream& os, char c) const;
      };

      struct printable: public raw
      {
	std::ostream&	encode(std::ostream& os, char c) const;
      };

      struct html
      {
	std::ostream&	begin(std::ostream& os) const;
	std::ostream&	end(std::ostream& os) const;

	std::ostream&	bold_begin(std::ostream& os) const;
	std::ostream&	bold_end(std::ostream& os) const;

	std::ostream&	encode(std::ostream& os, char c) const;
      };

    } // End of namespace fmt.

    namespace hex
    {

      namespace params
      {

	struct ahc4x4
	{
	  enum
	    {
	      bwidth	= 4,	// Block width.
	      lwidth	= 16,	// Line width.
	      p_addr	= true,	// Print addresses.
	      p_hex	= true,	// Print hexadecimal values.
	      p_char	= true, // Print character values.
	      endl	= true	// '\n' at end of lines.
	    };
	  static const std::string	bigsep;		// Wide space.
	  static const std::string	smallsep;	// Normal space.
	  static const std::string	colsep;		// Column separator.
	};

	// See above for attributes description.
	struct h1
	{
	  enum
	    {
	      lwidth	= 65536,
	      bwidth	= 65536,
	      p_addr	= false,
	      p_hex	= true,
	      p_char	= false,
	      endl	= false
	    };
	  static const std::string	bigsep;
	  static const std::string	smallsep;
	  static const std::string	colsep;
	};

	typedef ahc4x4	def;
	typedef h1	str;

      } // End of namespace tool::hex::params.

      template <class Params, class Format>
      std::string
      dump(const void*	data,
	   size_t		len,
	   const Format&	f);

      std::vector<uint8_t>
      read(const std::string& s);

    } // End of namespace wpl::tool::hex.

  } // End of namespace wpl::tool.

} // End of namespace wpl.

# include "hexdump.hxx"

#endif // ! WIPAL_TOOL_HEXDUMP_HH_
