/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef TOOL_ITERABLE_HXX_
# define TOOL_ITERABLE_HXX_

# include <algorithm>

# include "iterable.hh"

namespace wpl
{

  namespace tool
  {

    template <class B>
    typename iterable<B>::iterator
    iterable<B>::begin() const
    {
      return iterator (this->exact(), false);
    }

    template <class B>
    typename iterable<B>::iterator
    iterable<B>::end() const
    {
      return iterator (this->exact(), true);
    }

    template <class B>
    template <class O>
    void
    iterable<B>::operator () (const O& o) const
    {
      std::copy(this->begin(), this->end(), o);
    }

    template <class B>
    template <class F>
    void
    iterable<B>::for_each(F& f) const
    {
      const const_iterator e = end();

      for (const_iterator i = begin(); i != e; ++i)
	f(*i);
    }

    template <class B>
    template <class F>
    void
    iterable<B>::for_each(const F& f) const
    {
      const const_iterator e = end();

      for (const_iterator i = begin(); i != e; ++i)
	f(*i);
    }

  } // End of namespace wpl::tool.

} // End of namespace wpl.

#endif // ! TOOL_ITERABLE_HXX_
