/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef TOOL_LIST_OF_ITERABLES_HH_
# define TOOL_LIST_OF_ITERABLES_HH_

# include <queue>
# include <vector>

# include <wipal/tool/iterable.hh>
# include <wipal/tool/iterator.hh>

namespace wpl
{

  namespace tool
  {

    /*---------------------.
    | Forward declarations |
    `---------------------*/

    template <class T, class B>
    struct list_of_iterables;

    template <class T, class B1, class B2>
    struct list_of_iterables_iterator;

    /*--------------.
    | Virtual types |
    `--------------*/

    template <class T, class B, class Bottom>
    struct types< list_of_iterables_iterator<T, B, Bottom> >
    {
      typedef typename T::iterator::value_type		value_type;
      typedef list_of_iterables<T, B>			iterable_type;
    };

    template <class T, class B>
    struct types< list_of_iterables<T, B> >
    {
      typedef list_of_iterables_iterator<T, B, bottom>	iterator;
    };

    /*-------------.
    | Declarations |
    `-------------*/

    template <class T, class B, class Bottom = bottom>
    struct list_of_iterables_iterator:
      WP_INHERIT(public iterator, list_of_iterables_iterator<T, B, Bottom>)
    {
      typedef WP_GET_EXACT(Bottom, list_of_iterables_iterator<T, B, Bottom>)
	exact_type;
      typedef iterator<exact_type>		super_type;
      typedef WP_TYPE(value_type, exact_type)	value_type;
      typedef WP_TYPE(iterable_type, exact_type)	iterable_type;

      list_of_iterables_iterator(const iterable_type&, bool);

      bool		equal(const exact_type&) const;
      void		increment();

      const value_type&	get() const;
      const value_type*	get_ptr() const;

    private:
      typedef typename T::const_iterator	inner_iterator_type;

      std::queue<inner_iterator_type>	begins_;
      std::queue<inner_iterator_type>	ends_;
    };

    template <class T, class Bottom = tool::bottom>
    struct list_of_iterables: WP_INHERIT(public iterable,
					 list_of_iterables<T, Bottom>)
    {
      typedef WP_GET_EXACT(Bottom, list_of_iterables<T, Bottom>) exact_type;
      typedef iterable<exact_type>				 super_type;

      list_of_iterables();

      template <class L>
      list_of_iterables(const L& list);

      bool		empty() const;
      size_t		size() const;

      exact_type&	push_back(const T&);

      const T&		get(unsigned i) const;
      T&		get(unsigned i);

    private:
      std::vector<T>	list_;

      friend class list_of_iterables_iterator<T, Bottom, bottom>;
    };

  } // End of namespace wpl::tool.

} // End of namespace wpl.

# include "list_of_iterables.hxx"

#endif // ! TOOL_ITERABLE_HH_
