/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef TOOL_MAPPING_HH_
# define TOOL_MAPPING_HH_

# include <string>
# include <iostream>
# include <map>

# include <wipal/tool/datafile.hh>

namespace wpl
{

  namespace tool
  {

    /**
     * Mapping from designed to be stored into a database file.
     *
     * Load the mapping from the file at creation when the file exists.
     * Store it at deletion so it could be used later, or loaded in
     * a database using a LOAD DATA INFILE SQL command.
     *
     * @see addr_mapping
     *
     * @fixme FIXME: Document the @c Factory parameter.
     */
    template <class Key, class Value, class Factory>
    struct mapping : public datafile, private Factory
    {
      /// Construct the mapping given the database file's name.
      mapping(const std::string&	,
	      const Factory&		factory = Factory ());

      /**
       * Retrieve the value of a corresponding key.
       *
       * When the key already exists in the mapping, just return its
       * value.  Else, insert the (key, value) pair according to the
       * @c Factory template parameter.
       */
      const Value&	operator [] (const Key&);

    protected:
      /// Load a mapping from an @c std::istream.
      void		load(std::istream&);

    private:
      typedef std::map<Key, Value>	map_type;

      map_type	map_;
    };

  } // End of namespace wpl::tool.

} // End of namespace wpl.

# include "mapping.hxx"

#endif // ! TOOL_MAPPING_HH_
