/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef TOOL_MAPPING_HXX_
# define TOOL_MAPPING_HXX_

# include <utility>

# include "mapping.hh"

namespace wpl
{

  namespace tool
  {

    template <class K, class V, class F>
    mapping<K, V, F>::mapping(const std::string&	filename,
			      const F&			factory):
      datafile (filename),
      F (factory)
    {
      std::ifstream	f (filename.c_str());

      if (f)
	load(f);
    }

    template <class K, class V, class F>
    const V&
    mapping<K, V, F>::operator [] (const K& a)
    {
      const typename map_type::const_iterator  i = map_.find(a);

      if (i == map_.end())
	{
	  const std::pair<typename map_type::iterator, bool>  r =
	    map_.insert(std::make_pair(a, this->new_value(a, map_)));
	  assert(r.second);

	  const V&  v = r.first->second;

	  *this << v << '\t' << a << '\n';
	  return v;
	}
      return i->second;
    }

    template <class K, class V, class F>
    void
    mapping<K, V, F>::load(std::istream& in)
    {
      while (true)
	{
	  K	a;
	  V	i;

	  in >> i;
	  if (in.eof())
	    break;
	  if (not in)
	    throw std::invalid_argument ("Invalid mapping file");
	  in >> a;
	  if (not in or in.eof())
	    throw std::invalid_argument ("Invalid mapping file");
	  map_.insert(std::make_pair(a, i));
	  this->value_loaded(a, i, map_);
	}
    }

  } // End of namespace wpl::tool.

} // End of namespace wpl.

#endif // ! TOOL_MAPPING_HXX_
