/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef TOOL_SI_HXX_
# define TOOL_SI_HXX_

# include <cmath>
# include <iomanip>

# include "si.hh"

namespace wpl
{

  namespace tool
  {

    namespace internals
    {

      template <size_t OneUnitInBits>
      SI_unit<OneUnitInBits>::SI_unit(uint64_t count):
	bits_ (count * OneUnitInBits)
      {
      }

      template <size_t OneUnitInBits>
      template <size_t OneRHSUnitInBits>
      SI_unit<OneUnitInBits>::
      SI_unit(const SI_unit<OneRHSUnitInBits>& rhs): bits_ (rhs.bits())
      {
      }

      template <size_t OneUnitInBits>
      uint64_t
      SI_unit<OneUnitInBits>::bits() const
      {
	return bits_;
      }

      template <size_t OneUnitInBits>
      uint64_t
      SI_unit<OneUnitInBits>::bytes() const
      {
	return bits_ / 8;
      }

      template <size_t OneUnitInBits>
      std::string
      SI_unit<OneUnitInBits>::abbrev()
      {
	return abbrev_;
      }

      template <size_t OneUnitInBits>
      std::ostream&
      operator << (std::ostream& os, const SI_unit<OneUnitInBits>& unit)
      {
	const std::streamsize	default_p = os.precision();
	const std::ios::fmtflags	default_f = os.flags();
	const double		v = unit.bits() / OneUnitInBits;
	const std::string		u = SI_unit<OneUnitInBits>::abbrev();

	return os << std::fixed << std::setprecision (0)
		  << std::floor(v + 0.5) << ' ' << u
		  << std::setprecision (default_p)
		  << std::resetiosflags (~default_f)
		  << std::setiosflags (default_f);
      }

    } // End of namespace internals.

  } // End of namespace tool.

} // End of namespace wpl.

#endif // ! TOOL_SI_HXX_
