/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_STATS_BSS_STATS_HH_
# define WIFI_STATS_BSS_STATS_HH_

# include <map>

# include <wipal/wifi/stats/module.hh>
# include <wipal/wifi/addr.hh>
# include <wipal/wifi/essid.hh>

namespace wpl
{

  namespace wifi
  {

    namespace stats
    {

      struct bss_stats: public module
      {
	bss_stats(const void* other_modules = 0);

	template <class Frame>
	void
	account_beacon(const Frame&		frame,
		       const addr* const&	ap,
		       const addr* const&	bss,
		       const optional_string&	ssid,
		       const bool&		ibss);

	std::ostream&	print(std::ostream& o) const;

      private:
	struct single_sta_stats
	{
	  single_sta_stats();

	  typedef std::map<essid, unsigned>	essid_stats;

	  unsigned	total;
	  unsigned	independent;
	  essid_stats	essids;
	};

	struct single_bss_stats
	{
	  typedef std::map<addr, single_sta_stats>	sta_stats;

	  single_bss_stats();

	  bool		operator < (const single_bss_stats& rhs) const;
	  bool		operator == (const single_bss_stats& rhs) const;

	  unsigned	total_;
	  sta_stats	sta_stats_;
	};

	typedef std::map<wifi::addr, single_bss_stats>	internal_stats;

	unsigned	beacon_count_;
	unsigned	ap_count_;
	internal_stats	bss_stats_;
      };

    } // End of namespace wpl::wifi::stats.

  } // End of namespace wpl::wifi.

} // End of namespace wpl.

# include "bss_stats.hxx"

#endif // ! WIFI_STATS_BSS_STATS_HH_
