/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_STATS_DELIMITER_FRAMES_HH_
# define WIFI_STATS_DELIMITER_FRAMES_HH_

# include <wipal/wifi/stats/module.hh>

namespace wpl
{

  namespace wifi
  {

    namespace stats
    {

      struct delimiter_frames: public module
      {
	delimiter_frames(const void* other_modules);

	template <class Frame>
	void
	account_frame(const Frame&		frame,
		      const size_t&		len80211,
		      const type::frame_type&	type,
		      const unsigned&		subtype,
		      const bool&		retx,
		      const addr* const&	receiver,
		      const addr* const&	transmitter,
		      const bool&		from_ap);

	std::ostream&	print(std::ostream& o) const;
	void		restart();

      private:
	typedef std::pair<tool::microseconds, struct timeval>
							stamp_pair;
	typedef boost::optional<stamp_pair>		optional_stamp_pair;

	static
	std::ostream&
	print(std::ostream&			o,
	      const std::string&		name,
	      const optional_stamp_pair&	value,
	      const bool			phy);


	optional_stamp_pair	earliest_;
	optional_stamp_pair	latest_;
      };

    } // End of namespace wpl::wifi::stats.

  } // End of namespace wpl::wifi.

} // End of namespace wpl.

# include "delimiter_frames.hxx"

#endif // ! WIFI_STATS_DELIMITER_FRAMES_HH_
