/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_STATS_DELIMITER_FRAMES_HXX_
# define WIFI_STATS_DELIMITER_FRAMES_HXX_

# include <boost/date_time/posix_time/posix_time_duration.hpp>
# include <boost/date_time/posix_time/ptime.hpp>
# include <boost/date_time/posix_time/time_formatters.hpp>

# include <wipal/wifi/stats/delimiter_frames.hh>

namespace wpl
{

  namespace wifi
  {

    namespace stats
    {

      inline
      delimiter_frames::delimiter_frames(const void*)
      {
      }

      template <class Frame>
      void
      delimiter_frames::
      account_frame(const Frame&		frame,
		    const size_t&		,
		    const type::frame_type&	,
		    const unsigned&		,
		    const bool&			,
		    const addr* const&		,
		    const addr* const&		,
		    const bool&			)
      {
	const tool::microseconds&	phy_stamp = frame.microseconds();
	const struct timeval&		meta_stamp = frame.meta().ts;

	if (not earliest_ or earliest_->first > phy_stamp)
	  earliest_ = stamp_pair (phy_stamp, meta_stamp);

	if (not latest_ or latest_->first < phy_stamp)
	  latest_ = stamp_pair (phy_stamp, meta_stamp);
      }

      inline
      std::ostream&
      delimiter_frames::print(std::ostream& o) const
      {
	print(o, "earliest frame TOA", earliest_, false);
	print(o, "latest frame TOA", latest_, false);
	print(o, "smallest timestamp", earliest_, true);
	print(o, "greatest timestamp", latest_, true);
	return o << std::flush;
      }

      inline
      void
      delimiter_frames::restart()
      {
	earliest_ = latest_ = boost::none_t ();
      }

      std::ostream&
      delimiter_frames::print(std::ostream&			o,
			      const std::string&		name,
			      const optional_stamp_pair&	value,
			      const bool			phy)
      {
	o << name << ": ";
	if (not value)
	  o << "NA";
	else
	  if (phy)
	    o << value->first;
	  else
	    {
	      using namespace boost::gregorian;
	      using namespace boost::posix_time;

	      const ptime	t (date(1970, Jan, 01),
				   seconds (value->second.tv_sec) +
				   microseconds (value->second.tv_usec));

	      o << to_simple_string(t) << " UTC";
	    }
	return o << '\n';
      }

    } // End of namespace wpl::wifi::stats.

  } // End of namespace wpl::wifi.

} // End of namespace wpl.

#endif // ! WIFI_STATS_DELIMITER_FRAMES_HXX_
