/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_STATS_GAPS_HH_
# define WIFI_STATS_GAPS_HH_

# include <map>

namespace wpl
{

  namespace wifi
  {

    namespace stats
    {

      struct gaps: public module
      {
	gaps(const void*);

	template <class Frame>
	void
	account_frame(const Frame&		frame,
		      const size_t&		,
		      const type::frame_type&	ty,
		      const unsigned&		,
		      const bool&		,
		      const addr* const&	rx,
		      const addr* const&	tx,
		      const bool&		);

	void		account_gap(const unsigned& gap_length);

	std::ostream&	print(std::ostream& o) const;

      private:
	unsigned*			last_;

	std::map<addr, unsigned>	inter_frames_;

	enum
	  {
	    clear	= 0,
	    obstructed	= 1,
	  };

	unsigned	gaps_[2];
	unsigned	misses_[2];
      };

    } // End of namespace wpl::wifi::stats.

  } // End of namespace wpl::wifi.

} // End of namespace wpl.

# include "gaps.hxx"

#endif // ! WIFI_STATS_GAPS_HH_
