/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_STATS_GAPS_HXX_
# define WIFI_STATS_GAPS_HXX_

# include <cstring>
# include <numeric>

# include <wipal/wifi/stats/gaps.hh>

namespace wpl
{

  namespace wifi
  {

    namespace stats
    {

      inline
      gaps::gaps(const void*): last_ (0)
      {
	memset(gaps_, 0, sizeof (gaps_));
	memset(misses_, 0, sizeof (misses_));
      }

      template <class Frame>
      void
      gaps::account_frame(const Frame&			frame,
			  const size_t&			,
			  const type::frame_type&	ty,
			  const unsigned&		,
			  const bool&			,
			  const addr* const&		rx,
			  const addr* const&		tx,
			  const bool&			)
      {
	if (last_)
	  *last_ = 0;
	last_ = tx and ty != type::control? &inter_frames_[*tx]: 0;

	if (tx)
	  inter_frames_[*tx]++;
	if (rx and (not tx or *tx != *rx))
	  inter_frames_[*rx]++;
      }

      inline
      void
      gaps::account_gap(const unsigned& gap_length)
      {
	assert(last_);

	const unsigned	type = *last_ > 1? obstructed : clear;

	++gaps_[type];
	misses_[type] += gap_length;
      }

      inline
      std::ostream&
      gaps::print(std::ostream& o) const
      {
	const double	gap_total =
	  std::accumulate(gaps_, gaps_ + sizeof (gaps_) / sizeof (*gaps_), 0);

	const double	miss_total =
	  std::accumulate(misses_,
			  misses_ + sizeof (misses_) / sizeof (*misses_), 0);

	return
	  o << "clear gaps: " << gaps_[clear]
	    << "\nclear gaps ratio: " << (gaps_[clear] / gap_total)
	    << "\nobstructed gaps: " << gaps_[obstructed]
	    << "\nobstructed gaps ratio: " << (gaps_[obstructed] / gap_total)
	    << "\nclear miss count: " << misses_[clear]
	    << "\nclear miss count ratio: " << (misses_[clear] / miss_total)
	    << "\nobstructed miss count: " << misses_[obstructed]
	    << "\nobstructed miss count ratio: "
	    <<                 (misses_[obstructed] / miss_total) << std::endl;
      }

    } // End of namespace wpl::wifi::stats.

  } // End of namespace wpl::wifi.

} // End of namespace wpl.

#endif // ! WIFI_STATS_GAPS_HXX_
