/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_STATS_GROWTH_HH_
# define WIFI_STATS_GROWTH_HH_

# include <wipal/wifi/stats/epoch_tracker.hh>
# include <wipal/wifi/stats/module.hh>

namespace wpl
{

  namespace wifi
  {

    namespace stats
    {

      template <class T, unsigned EpochLength>
      struct growth: public abstract_stats
      {
	void		restart();
	void		update(const tool::microseconds& timestamp);
	void		update(const T& elt);
	std::ostream&	print(std::ostream& o) const;

      private:
	struct epoch_data
	{
	  epoch_data();

	  unsigned	total_count;
	  unsigned	current_count;
	};

	std::set<T>				elements_;
	std::set<T>				current_elements_;
	epoch_tracker<epoch_data, EpochLength>	tracker_;
      };

      template <unsigned Epoch, bool OnlyAccountTxer = false>
      struct addr_growth: public module
      {
	addr_growth(const void* other_modules = 0);

	void		account_addr(unsigned, const addr& a);

	template <class Frame>
	void
	account_frame(const Frame&		frame,
		      const size_t&		len80211,
		      const type::frame_type&	type,
		      const unsigned&		subtype,
		      const bool&		retx,
		      const addr* const&	receiver,
		      const addr* const&	transmitter,
		      const bool&		from_ap);

	std::ostream&	print(std::ostream& os) const;
	void		restart();

      protected:
	typedef growth<addr, Epoch>	growth_type;

	growth_type			growth_;
	bool				ignore_;
      };

      template <unsigned Epoch>
      struct beacon_items_growth: public module
      {
	beacon_items_growth(const void* other_modules = 0);

	template <class Frame>
	void
	account_beacon(const Frame&		frame,
		       const addr* const&	ap,
		       const addr* const&	bss,
		       const optional_string&	ssid,
		       const bool&		ibss);

	template <class Frame>
	void
	account_frame(const Frame&		frame,
		      const size_t&		len80211,
		      const type::frame_type&	type,
		      const unsigned&		subtype,
		      const bool&		retx,
		      const addr* const&	receiver,
		      const addr* const&	transmitter,
		      const bool&		from_ap);

	std::ostream&	print(std::ostream& os) const;
	void		restart();

      private:
	typedef growth<addr,	    Epoch>	bssid_growth;
	typedef growth<addr,	    Epoch>	ibssid_growth;
	typedef growth<std::string, Epoch>	ssid_growth;
	typedef growth<addr,	    Epoch>	ap_growth;

	ap_growth		ap_;
	bssid_growth		bssid_;
	ibssid_growth		ibssid_;
	ssid_growth		ssid_;
	bool			ignore_;
      };

    } // End of namespace wpl::wifi::stats.

  } // End of namespace wpl::wifi.

} // End of namespace wpl.

# include "growth.hxx"

#endif // ! WIFI_STATS_GROWTH_HH_
