/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_STATS_ON_OFF_DATA_HH_
# define WIFI_STATS_ON_OFF_DATA_HH_

# include <wipal/wifi/stats/module.hh>

namespace wpl
{

  namespace wifi
  {

    namespace stats
    {

      struct on_off_data: public module
      {
	on_off_data(const void* other_modules = 0);

	template <class Frame>
	void
	account_frame(const Frame&		frame,
		      const size_t&		len80211,
		      const type::frame_type&	type,
		      const unsigned&		subtype,
		      const bool&		retx,
		      const addr* const&	receiver,
		      const addr* const&	transmitter,
		      const bool&		from_ap);

	void			restart();
	std::ostream&		print(std::ostream& os) const;

	tool::microseconds	time_on(const addr& sta) const;

      private:

	struct state
	{
	  enum
	    {
	      /**
	       * Make STA inactive for more than @c lifetime
	       * microseconds expire.
	       *
	       * Set to 3 minute.  Rationale: it seems many APs use
	       * this value as inactivity timer before they
	       * disassociate STAs.
	       */
	      lifetime = 3 * 60 * 1000 * 1000
	    };

	  state(const tool::microseconds& first_seen);

	  tool::microseconds			last_seen;
	  std::vector<tool::microseconds>	events;
	};

	typedef std::pair<unsigned, addr>	state_id;
	typedef std::map<state_id, state>	states;

	unsigned			trace_;
	std::vector<tool::microseconds>	last_stamps_;
	states				states_;
      };


    } // End of namespace wpl::wifi::stats.

  } // End of namespace wpl::wifi.

} // End of namespace wpl.

# include "on_off_data.hxx"

#endif // ! WIFI_STATS_ON_OFF_DATA_HH_
