/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_STATS_STA_COUNTERS_HH_
# define WIFI_STATS_STA_COUNTERS_HH_

# include <wipal/wifi/stats/on_off_data.hh>

namespace wpl
{

  namespace wifi
  {

    namespace stats
    {

      struct sta_counters: public module
      {
	sta_counters(const on_off_data* ood);

	template <class Frame>
	void
	account_frame(const Frame&		frame,
		      const size_t&		len80211,
		      const type::frame_type&	type,
		      const unsigned&		subtype,
		      const bool&		retx,
		      const addr* const&	receiver,
		      const addr* const&	transmitter,
		      const bool&		from_ap);
	std::ostream&	print(std::ostream& o) const;

      private:
	struct counters
	{
	  counters();

	  unsigned	count;
	  uint64_t	size;
	};

	typedef std::map<addr, counters>	states;

	struct size_less
	{
	  bool	operator () (const states::const_iterator& lhs,
			     const states::const_iterator& rhs) const;
	};

	struct avg_rate_less
	{
	  avg_rate_less(const on_off_data* ood);

	  bool	operator () (const states::const_iterator& lhs,
			     const states::const_iterator& rhs) const;

	private:
	  const on_off_data*	ood_;
	};

	struct time_on_less
	{
	  time_on_less(const on_off_data* ood);

	  bool	operator () (const states::const_iterator& lhs,
			     const states::const_iterator& rhs) const;

	private:
	  const on_off_data*	ood_;
	};

	template <class Set>
	std::ostream&
	print(std::ostream&		os,
	      const std::string&	name,
	      const Set&		set) const;

	template <class Less>
	std::set<states::const_iterator, Less>
	sorted_states(const Less& less = Less ()) const;

	states			states_;
	const on_off_data*	ood_;
      };

    } // End of namespace wpl::wifi::stats.

  } // End of namespace wpl::wifi.

} // End of namespace wpl.

# include "sta_counters.hxx"

#endif // ! WIFI_STATS_STA_COUNTERS_HH_
