/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_STATS_TFI_PLOT_HXX_
# define WIFI_STATS_TFI_PLOT_HXX_

# include <wipal/wifi/stats/tfi_plot.hh>

namespace wpl
{

  namespace wifi
  {

    namespace stats
    {

      inline
      tfi_plot::tfi_plot(const void*):
	tfi_ (boost::extents[100 / score_step + 1][load_max / load_step]),
	epoch_start_ (0),
	total_ (0),
	missed_ (0)
      {
      };

      template <class Frame>
      void
      tfi_plot::account_frame(const Frame&		frame,
			      const size_t&		,
			      const type::frame_type&	,
			      const unsigned&		,
			      const bool&		,
			      const addr* const&	,
			      const addr* const&	,
			      const bool&		)
      {

	const tool::microseconds&	stamp = frame.microseconds();

	if (0 == epoch_start_)
	  epoch_start_ = stamp;
	else
	  {
	    if (stamp < epoch_start_)
	      {
		std::cerr << frame << ": Out of sequence." << std::endl;
		return;
	      }

	    const tool::microseconds	  dt = stamp - epoch_start_;

	    if (dt > epoch_length)
	      {
		const tool::microseconds  el (epoch_length);
		const unsigned		  k = dt.get_div_by(epoch_length);

		end_epoch(epoch_start_ + el * k);
	      }
	  }
	++total_;
      }

      inline
      void
      tfi_plot::account_gap(const unsigned& gap_length)
      {
	missed_ += gap_length;
      }

      inline
      void
      tfi_plot::account_miss()
      {
	++missed_;
      }

      inline
      std::ostream&
      tfi_plot::print(std::ostream& o) const
      {
	o << "begin T-Fi plot\n";

	for (unsigned i = 0; i < 100 / score_step + 1; ++i)
	  for (unsigned k = 0; k < 2; ++k)
	    {
	      if (i or k)
		o << '\n';

	      unsigned		x = 0;
	      const double	y = (i + k) * score_step / 100.l;

	      for (unsigned j = 0; j < load_max / load_step; ++j)
		{
		  o << "  " << x << '\t' << y << '\t' << tfi_[i][j] << '\n';
		  x += load_step;
		  o << "  " << x << '\t' << y << '\t' << tfi_[i][j] << '\n';
		}
	    }
	return o << "end T-Fi plot" << std::endl;
      }

      inline
      void
      tfi_plot::restart()
      {
	end_epoch(0);
      }

      inline
      void
      tfi_plot::end_epoch(const tool::microseconds& new_epoch_start)
      {
	if (total_ + missed_)
	  {
	    const double load  = total_ + missed_;
	    const double score = total_ / load;

	    if (load < load_max)
	      ++tfi_[unsigned (score * 100 / score_step)][load / load_step];
	  }
	epoch_start_ = new_epoch_start;
	total_ = 0;
	missed_ = 0;
      }


    } // End of namespace wpl::wifi::stats.

  } // End of namespace wpl::wifi.

} // End of namespace wpl.

#endif // ! WIFI_STATS_TFI_PLOT_HXX_
