/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_TRACES_MERGE_HXX_
# define WIFI_TRACES_MERGE_HXX_

# include "merge.hh"

# include <stdexcept>

# include <wipal/pcap/ostream.hh>
# include <wipal/pcap/list.hh>
# include <wipal/filter/merge.hh>

namespace wpl
{

  namespace wifi
  {

    /// Internal helpers for merge().
    namespace merge_helpers
    {

      struct dump
      {
	dump(pcap::ostream& os): os_ (os)
	{
	}

	template <class Merger>
	void
	operator () (const Merger& m)
	{
	  m.for_each(os_);
	}

      private:
	pcap::ostream&	os_;
      };

    } // End of namespace wpl::wifi::merge_helpers.

    template <class UniqueId, class PhyHeader,
	      template <class, class, class> class Intersector,
	      template <class, class, class, class> class Merger,
	      class T1, class T2>
    void
    merge(T1&			t1,
	  T2&			t2,
	  const std::string&	output,
	  opt::list&		options)
    {
      pcap::ostream		os (output, t2);
      merge_helpers::dump	dump (os);

      filter::provide_merge<UniqueId, PhyHeader, Intersector, Merger>
	(t1.begin(), t1.end(), t2.begin(), t2.end(), options, dump);
    }

  } // End of namespace wpl::wifi.

} // End of namespace wpl.

#endif // ! WIFI_TRACES_MERGE_HXX_
