/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_TRACES_MERGE_AND_SYNCHRONIZE_HXX_
# define WIFI_TRACES_MERGE_AND_SYNCHRONIZE_HXX_

# include "merge_and_synchronize.hh"

# include <wipal/wifi/traces/merge.hh>

namespace wpl
{

  namespace wifi
  {

    template <class UniqueID, class PhyHeader,
	      template <class, class, class> class Intersector,
	      template <class, class, class, class> class Merger,
	      class C>
    std::string
    merge_and_synchronize(C& traces, opt::list& options)
    {
      typedef pcap::list<>*				list_ptr;
      typedef std::set< std::pair<unsigned, unsigned> >	blist_type;

      if (not options.count("ref_blacklist"))
	options.add("ref_blacklist", blist_type ());

      std::string		out = merge<UniqueID, PhyHeader,
					    Intersector, Merger>(traces,
								 options);
      pcap::file<>		ref (out);

      unsigned	i = 0;
      BOOST_FOREACH(const typename C::value_type& t, traces)
	{
	  std::string		outp = ("sync-" +
					boost::lexical_cast<std::string> (++i));
	  pcap::ostream		os (outp, t);
	  merge_helpers::dump	dump (os);

	  filter::provide_time_adjuster<UniqueID,
					PhyHeader,
					Intersector>(t.begin(),   t.end(),
						     ref.begin(), ref.end(),
						     options, dump);
	}
      return out;
    }

  } // End of namespace wpl::wifi.

} // End of namespace wpl.

#endif // ! WIFI_TRACES_MERGE_AND_SYNCHRONIZE_HXX_
