/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_ADDR_TIMESTAMP_ID_HH_
# define WIFI_ADDR_TIMESTAMP_ID_HH_

# include <boost/optional.hpp>

# include <wipal/wifi/unique_id/unique_id.hh>
# include <wipal/wifi/timestamp.hh>
# include <wipal/pcap/pkt.hh>
# include <wipal/wifi/addr_mapping.hh>
# include <wipal/wifi/dissector/dissector.hh>
# include <wipal/wifi/unique_id/helper_hooks.hh>
# include <wipal/wifi/mgt.hh>

namespace wpl
{

  namespace wifi
  {

    namespace internals
    {

      struct dst_tmp_tag {};
      struct src_tmp_tag {};
      struct bss_tmp_tag {};

    } // End of namespace wpl::wifi::internals.

# define WP_define_addr_tmp_id(AddrName, AddrIdx)			\
									\
    typedef								\
      unique_id<internals::adapter<unsigned,				\
				   internals::AddrName ## _tmp_tag>,	\
		timestamp>						\
      AddrName ## _tmp_id;						\
									\
    template <>								\
    struct unique_id_factory<AddrName ## _tmp_id>			\
    {									\
      template <class HeaderType, class S>				\
      boost::optional<AddrName ## _tmp_id>				\
      build(const pkt::packet<S>& p, addr_mapping& mapping)		\
      {									\
	WP_UNIQUE_ID_DISSECT(p, d, internals::id_helper_hooks, h);	\
									\
	if (const timestamp* t = h.timestamp_ptr())			\
	  {								\
	    const addr* a = h.addr_ptr(AddrIdx);			\
									\
	    assert(a);							\
	    return AddrName ## _tmp_id (p.id(), mapping[*a], *t);	\
	  }								\
	return boost::none_t ();					\
      }									\
    };

    WP_define_addr_tmp_id(dst, mgt::header::da)
    WP_define_addr_tmp_id(src, mgt::header::sa)
    WP_define_addr_tmp_id(bss, mgt::header::bssid)

# undef WP_define_addr_tmp_id

  } // End of namespace wpl::wifi.

} // End of namespace wpl.

#endif // ! WIFI_ADDR_TIMESTAMP_ID_HH_
