/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_ID_HELPER_HOOKS_HXX_
# define WIFI_ID_HELPER_HOOKS_HXX_

# include "helper_hooks.hh"

# include <wipal/wifi/frame.hh>

namespace wpl
{

  namespace wifi
  {

    namespace internals
    {

      inline
      id_helper_hooks::id_helper_hooks():
	addrs_ (0),
	seq_ctl_ (0),
	timestamp_ (0)
      {
      }


      inline
      const addr*
      id_helper_hooks::addr_ptr(unsigned i) const
      {
	return addrs_ ? addrs_ + i : 0;
      }

      inline
      const uint16_t*
      id_helper_hooks::seq_ctl_ptr() const
      {
	return seq_ctl_;
      }

      inline
      const timestamp*
      id_helper_hooks::timestamp_ptr() const
      {
	return timestamp_;
      }

      inline
      void
      id_helper_hooks::beacon_hook(const mgt::header* h, size_t caplen)
      {
	unique_frame_hook(h, caplen);
      }

      inline
      void
      id_helper_hooks::probe_resp_hook(const mgt::header* h, size_t caplen)
      {
	if (not control_flag(3, h)) // Ignore retransmitted frames.
	  unique_frame_hook(h, caplen);
      }

      inline
      void
      id_helper_hooks::unique_frame_hook(const mgt::header* h, size_t caplen)
      {
	if (caplen < sizeof (*h) + sizeof (*timestamp_))
	  return;

	addrs_ = h->addrs;
	seq_ctl_ = &(h->seq_ctl);
	timestamp_ = reinterpret_cast<const timestamp*> (h->body);
      }

    } // End of namespace wpl::wifi::internals.

  } // End of namespace wpl::wifi.

} // End of namespace wpl.

#endif // ! WIFI_ID_HELPER_HOOKS_HXX_

