/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_SEQCTL_SOURCE_BSSID_TIMESTAMP_ID_HXX_
# define WIFI_SEQCTL_SOURCE_BSSID_TIMESTAMP_ID_HXX_

# include "seqctl_source_bssid_timestamp.hh"

# include <wipal/wifi/dissector/dissector.hh>
# include <wipal/wifi/unique_id/helper_hooks.hh>

namespace wpl
{

  namespace wifi
  {

    template <class HeaderType, class S>
    boost::optional<seq_src_bss_tmp_id>
    unique_id_factory<seq_src_bss_tmp_id>::
    build(const pkt::packet<S>& p, addr_mapping& mapping)
    {
      WP_UNIQUE_ID_DISSECT(p, d, internals::id_helper_hooks, h);

      if (const timestamp* tstamp = h.timestamp_ptr())
	{
	  const addr*		sa = h.addr_ptr(mgt::header::sa);
	  const addr*		bssid = h.addr_ptr(mgt::header::bssid);
	  const uint16_t*	seqctl = h.seq_ctl_ptr();

	  assert(sa and bssid and seqctl);

	  return seq_src_bss_tmp_id (p.pcap_header()->id,
				     *seqctl,
				     mapping[*sa],
				     mapping[*bssid],
				     *tstamp);
	}
      return boost::none_t ();
    }

  } // End of namespace wpl::wifi.

} // End of namespace wpl.

#endif // ! WIFI_SEQCTL_SOURCE_BSSID_TIMESTAMP_ID_HXX_

