import re
import datetime

def parse(files):

    def adjust(events, statfile, offset, duration):
        """Normalize events' timestamps for file statfile according to
        offset and duration."""
        for sta, evs in events.items():
            if statfile in evs:
                evs[statfile] = \
                    map(lambda (t, k): (float (t + offset) / duration, k),
                        evs[statfile])

    def us(tdelta):
        """Convert  a timedelta to a microsecond count."""
        return ((tdelta.days * 24 * 60 * 60) + tdelta.seconds) * 1000000 + \
            tdelta.microseconds

    xdigit = "[0-9a-fA-F]"
    macaddr = "(%s{2}:){5}%s{2}" % (xdigit, xdigit)
    date_format = "%Y-%b-%d %H:%M:%S"

    erl_ro = re.compile("earliest frame TOA: (.*)\.(\d+) UTC")
    smt_ro = re.compile("smallest timestamp: (\d+)")
    grt_ro = re.compile("greatest timestamp: (\d+)")
    bgn_ro = re.compile("\s*begin ON/OFF T\d+ STA (%s)" % macaddr)
    end_ro = re.compile("\s*end ON/OFF T\d+ STA (%s)" % macaddr)
    evt_ro = re.compile("\s*(\d+)\s+([01])")

    start    = dict ()
    duration = dict ()
    events   = dict ()

    for statfile in files:
        smt = None # Smallest timestamp.
        grt = None # Greatest timestamp.
        cur = None # Current STA.

        for line in open(statfile):
            if smt is None or grt is None or statfile not in start:
                m = erl_ro.match(line)
                if m:
                    start[statfile] =                                          \
                        datetime.datetime.strptime (m.group(1), date_format) + \
                        datetime.timedelta (microseconds = int (m.group(2)))
                    continue

                m = smt_ro.match(line)
                if m:
                    smt = int (m.group(1))
                else:
                    m = grt_ro.match(line)
                    if m:
                        grt = int (m.group(1))
                if statfile not in duration and \
                        smt is not None and grt is not None:
                    duration[statfile] = grt - smt
                continue

            if cur is None:
                m = bgn_ro.match(line)
                if m:
                    cur = m.group(1)
                    if cur in events:
                        events[cur][statfile] = []
                    else:
                        events[cur] = { statfile: [] }
                continue

            m = end_ro.match(line)
            if m:
                cur = None
                continue

            m = evt_ro.match(line)
            if not m:
                raise "Invalid event: '%s'" % line
            stamp, kind = m.group(1, 2)
            events[cur][statfile].append((int (stamp) - smt, int (kind)))

    min_start = min(start.values())
    offset = dict (((sf, us(start[sf] - min_start)) for sf in files))
    max_dur =max((offset[sf] + duration[sf] for sf in files))
    for statfile in files:
        adjust(events, statfile, offset[statfile], max_dur)

    return max_dur, events
