/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#include <stdexcept>
#include <iostream>
#include <cstdlib>

#include <wipal/tool/args.hh>
#include <wipal/pcap/list.hh>
#include <wipal/pcap/ostream.hh>
#include <wipal/filter/non_noisy_prism.hh>

int
main(int argc, char *const argv[])
{
  try
    {
      using namespace wpl;
      namespace def = args::def;

      opt::list opts ((opt::name ("internals") -= opt::list
		       (( opt::name ("ignore_noisy_prism") -=
						 def::ignore_noisy_prism()))));

      args::parse(argc, argv, opts, args::count (1), args::count (1));

      const args::input&	input = opt::get<args::input>(opts["proc"]
							      ["input"]);
      pcap::list<>		traces (input[0]);
      const pkt::type		lt = traces.type();

      args::check_options(opts, lt);
      traces.expect(lt);

      const args::output&	output = opt::get<args::output>(opts["proc"]
								["output"]);
      pcap::ostream		dump (output[0], traces.get(0));

      if (opt::get<bool>(opts["internals"]["ignore_noisy_prism"]))
	{
	  typedef filter::non_noisy_prism<pcap::list<>::const_iterator>	nnp;

	  nnp			filter (traces.begin(), traces.end());

	  filter.for_each(dump);
	}
      else
	{
	  traces.for_each(dump);
	}
    }
  catch (std::exception& e)
    {
      std::cerr << "ERROR: " << e.what() << '.' << std::endl;
      return EXIT_FAILURE;
    }
}
