/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#include <iostream>
#include <stdexcept>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>

#include <wipal/tool/args.hh>
#include <wipal/wifi/stats/stats.hh>
#include <wipal/pcap/list.hh>
#include <wipal/filter/timetracker.hh>
#include <wipal/filter/non_noisy_prism.hh>

int
main(int argc, char *const argv[])
{
  try
    {
      using namespace wpl;
      namespace def = args::def;

      typedef tool::end::endianness	endianness;
      typedef wifi::stats::stats	stats;

      opt::list opts ((opt::name ("internals") -= opt::list
		       ((opt::name ("endianness") -= def::endianness(),
			 opt::name ("ignore_noisy_prism") -=
						     def::ignore_noisy_prism(),
			 opt::name ("phy_tstamps") -= def::phy_tstamps()))));

      args::parse(argc, argv, opts, args::arbitrary, args::none);

      stats	st;
      bool	first = true;

      BOOST_FOREACH(pcap::list<> t,
		    opt::get<args::input>(opts["proc"]["input"]))
	{
	  if (first)
	    first = false;
	  else
	    st.restart();

	  if (t.empty())
	    continue;

	  const pkt::type lt = t.type();
	  args::check_options(opts, lt);
	  args::ensure_80211(lt);
	  t.expect(lt);

	  const endianness&	end = opt::get<endianness>(opts["internals"]
							   ["endianness"]);
	  const bool		pcapts = not opt::get<bool>(opts["internals"]
							    ["phy_tstamps"]);
	  switch (lt)
	    {

#define WP_case(R, Unused, LT)						\
	    case pkt::BOOST_PP_TUPLE_ELEM(2, 0, LT):			\
	      {								\
		using boost::bind;					\
									\
		typedef BOOST_PP_TUPLE_ELEM(2, 1, LT)	phy_header;	\
		typedef pcap::list<>::const_iterator	iterator;	\
		typedef filter::timetracker<iterator, phy_header>	\
							timetracker;	\
									\
		timetracker	trk (t.begin(), t.end(), end, pcapts);	\
									\
		if (opt::get<bool>(opts["internals"]["ignore_noisy_prism"])) \
		  {							\
		    typedef timetracker::const_iterator		it;	\
		    typedef filter::non_noisy_prism<it>		nnp;	\
		    typedef nnp::const_iterator::value_type	frame;	\
									\
		    nnp	n (trk.begin(), trk.end());			\
									\
		    n.for_each(bind(&stats::account<phy_header, frame>, \
				    boost::ref(st), _1));		\
		  }							\
		else							\
		  {							\
		    typedef						\
		      timetracker::const_iterator::value_type		\
		      frame;						\
									\
		    trk.for_each(bind(&stats::account<phy_header, frame>, \
				      boost::ref(st), _1));		\
		  }							\
	      }								\
	    break;

	      BOOST_PP_SEQ_FOR_EACH(WP_case, ~, WP_LINKTYPES)

	    default:
		args::unsupported_linktype(lt);
	      break;
	    }
	}

      std::cout << st;
    }
  catch (std::exception& e)
    {
      std::cerr << "ERROR: " << e.what() << '.' << std::endl;
      return EXIT_FAILURE;
    }
}
