/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#include <iostream>
#include <cstdlib>
#include <stdexcept>

#include <wipal/phy/prism_header.hh>
#include <wipal/pcap/stream.hh>

#include "wifi_stats.hh"
#include "config.hh"

namespace
{

  struct context
  {
    context(bool prism_header):
      frames (),
      prism_header (prism_header)
    {
    }

    template <class T>
    void
    operator () (const wpl::pkt::packet<T>& d)
    {
      if (prism_header)
	frames.update(d.meta(),
		      static_cast<const wpl::prism::header*> (d.bytes()));
      else
	frames.update(d.meta(), d.bytes());
    }

    std::ostream&
    dump(std::ostream& os) const
    {
      return os << frames;
    }

  private:
    stats::frames	frames;
    bool		prism_header;
  };

  std::ostream&
  operator << (std::ostream& os, const context& c)
  {
    return c.dump(os);
  }

} // End of anonymous namespace.

int
main(int argc, char *const argv[])
{
  if (argc > 2)
    {
      std::cerr << "Usage:\t"  << argv[0] << " [trace]" << std::endl;
      return EXIT_FAILURE;
    }

  try
    {
      wpl::pcap::file<>	d (argc == 2 ? argv[1] : "-");
      bool		prism_header (true);

      if (d.type() != wpl::pkt::PRISM_HEADER)
	{
	  if (d.type() != wpl::pkt::IEEE802_11)
	    throw std::invalid_argument("invalid pcap datalink");
	  prism_header = false;
	}

      context		c (prism_header);

      for (wpl::pcap::file<>::const_iterator i = d.begin(); i != d.end(); ++i)
	c(*i);
      std::cout << c;
    }
  catch (const std::exception& e)
    {
      std::cerr << "ERROR: " << e.what() << '.' << std::endl;
      return EXIT_FAILURE;
    }
}
