#include <boost/filesystem.hpp>

#include <iostream>
#include <cstdlib>

#include <wipal/filter/anon_policies.hh>

#define WP_CHECK(Cond)					\
  do							\
    {							\
      if (not (Cond))					\
	{						\
	  std::cerr << #Cond " failed." << std::endl;	\
	  abort();					\
	}						\
    }							\
  while (false)

int
main()
{
  using namespace wpl;
  namespace fs = boost::filesystem;

  typedef wifi::anon::prefix_preserving_mac_policy		mac_policy;
  typedef wifi::anon::prefix_preserving_essid_policy	essid_policy;

  fs::path	mp_path ("anon_policies-MAC.tmp");
  fs::path	ep_path ("anon_policies-ESSID.tmp");

  if (exists(mp_path))
    remove(mp_path);
  if (exists(ep_path))
    remove(ep_path);

  mac_policy	mp (mp_path.file_string());
  essid_policy	ep (ep_path.file_string());

  const wifi::addr	m[]  = { std::string ("42:42:42:51:51:51"),
				 std::string ("42:42:42:51:51:51"),
				 std::string ("42:42:42:51:51:42"),
				 std::string ("51:51:51:51:51:42") };
  const wifi::addr	am[] = { mp[m[0]], mp[m[1]], mp[m[2]], mp[m[3]] };

  const wifi::essid	e[]  = { std::string ("ab1"),
				 std::string ("ab1"),
				 std::string ("ab2"),
				 std::string ("bb1") };
  const wifi::essid	ae[] = { ep[e[0]], ep[e[1]], ep[e[2]], ep[e[3]] };

  /*-----------------.
  | MAC policy tests |
  `-----------------*/

  WP_CHECK(am[0] == am[1]);
  WP_CHECK(am[0] != am[2]);
  WP_CHECK(am[0] != am[3]);
  WP_CHECK(am[2] != am[3]);

  for (size_t i = 0; i < sizeof (am) / sizeof (*am); ++i)
    for (size_t j = 0; j < 3; ++j)
      WP_CHECK(am[i][j] == m[i][j]);

  // FIXME: The following test actually fails with a probability of 1 / 2^24.
  WP_CHECK(am[2][3] != am[3][3] or
	   am[2][4] != am[3][4] or
	   am[2][5] != am[3][5]);

  /*------------.
  | ESSID tests |
  `------------*/

  WP_CHECK(ae[0] == ae[1]);
  WP_CHECK(ae[0] != ae[2]);
  WP_CHECK(ae[0] != ae[3]);
  WP_CHECK(ae[2] != ae[3]);

  for (size_t i = 0; i < 2; ++i)
    WP_CHECK(ae[0][i] == ae[2][i]);
  WP_CHECK(ae[0][0] != ae[3][0]);

  remove(mp_path);
  remove(ep_path);
}

