/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#include <iostream>
#include <string>
#include <cstdlib>
#include <cstring>

#include <wipal/pcap/stream.hh>
#include <wipal/pcap/ostream.hh>

int
main()
{
  std::cerr << "WARNING: This test requires several giga-bytes "
    " available on your disk." << std::endl
	    << "WARNING: Set the DONT_RUN_THE_BIG_PCAP_TEST environment "
    "variable to disable it." << std::endl;

  if (getenv("DONT_RUN_THE_BIG_PCAP_TEST"))
    return EXIT_SUCCESS;

  const std::string	pcapname = "big";
  const size_t		npackets = (1 << 26);
  const size_t		snaplen = 16;
  char			junk[snaplen];

  std::cout << "Creating a big Pcap file..." << std::endl;
  {
    wpl::pkt::metadata	meta (0, 0, 0, snaplen, snaplen);

    memset(junk, 0xd0, snaplen);

    wpl::pcap::ostream		os (pcapname, wpl::pkt::PRISM_HEADER);

    for (unsigned i = 0; i < npackets; ++i)
      {
	++meta.ts.tv_sec;
	os.write(meta, junk);
      }
  }

  std::cout << "Checking whether we are capable of opening it or not..."
	    << std::endl;
  wpl::pcap::file<>	test (pcapname);

  typedef wpl::pcap::file<>::const_iterator	iterator;

  // Regression test regarding pcapxx::descriptor<>::iterator's copy
  // constructor and assignment operator.
  {
    iterator	i = test.begin();
    iterator	j (i);
    bool	t1 = i->meta().id != 1;
    bool	t2 = j->meta().id != 1;
    if (t1)
      std::cerr << "ERROR: Regression failed on assignment operator."
		<< std::endl;
    if (t2)
      std::cerr << "ERROR: Regression failed on copy constructor." << std::endl;
    if (t1 or t2)
      return EXIT_FAILURE;
  }


  std::cout << "Integrity check..." << std::endl;
  {
    int n = 0;

    for (iterator i = test.begin(); i != test.end(); ++i)
      {
	if (i->meta().ts.tv_sec != ++n)
	  {
	    std::cerr << "ERROR: Invalid PCAP header for packet " << n << '.'
		      << std::endl;
	    return EXIT_FAILURE;
	  }
	if (memcmp(i->bytes(), junk, sizeof (junk)))
	  {
	    std::cerr << "ERROR: Invalid PCAP bytes for packet " << n << '.'
		      << std::endl;
	    return EXIT_FAILURE;
	  }
      }
    if (n != int (npackets))
      {
	std::cerr << "ERROR: Got " << n << " packets, expected " << npackets
		  << '.' << std::endl;
	return EXIT_FAILURE;
      }
  }
}
