/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#include <iostream>
#include <vector>
#include <cmath>
#include <cstdlib>

#include <wipal/tool/linear_regression.hh>

template <class T>
bool
near(const T& a, const T& b)
{
  const T epsilon (0.1);

  return std::abs(a - b) <= epsilon;
}

int
main()
{
  typedef unsigned				input_type;
  typedef float					impl_type;
  typedef wpl::tool::lr::pair<impl_type>	adapter_type;
  typedef std::pair<input_type, input_type>	input_pair;
  typedef std::pair<impl_type, impl_type>	output_pair;

  const input_pair		p1 (10, 10);
  const input_pair		p2 (20, 16);
  const input_pair		p3 (30, 20);

  std::vector<input_pair>	v;
  v.push_back(p1);
  v.push_back(p2);
  v.push_back(p3);

  const output_pair r =
    wpl::tool::linear_regression<impl_type, adapter_type>(v.begin(), v.end());

  if (getenv("DEBUG"))
    std::cerr << "a = " << r.first << ", b = " << r.second << std::endl;

  if (not near(r.first, impl_type (0.5)) or
      not near(r.second, impl_type (5.33)))
    abort();
}
