#! /usr/bin/env python

from scapy.layers.dot11 import *
import random
import sys
import struct

lhs     = list ()
rhs     = list ()
inter   = list ()
union   = list ()

lhs_clock = random.randint(0, 1000000);
rhs_clock = random.randint(0, 1000000);

lhs_clock_speed = random.random();
rhs_clock_speed = random.random();


def append(f, l, t):
    f2 = f.copy()
    if type(f2) == PrismHeader:
        f2.mactime = t
        f2.hosttime = t / 10
    elif type(f2) == RadioTap:
        f2.present = 1 # TSFT
        f2.notdecoded = struct.pack("Q", t) # Won't work on big-endian hosts.
    else:
        f2.time = t / float (1000000)
    l.append(f2)

def gen(f):
    global rhs, lhs, inter, union, \
        lhs_clock, rhs_clock, lhs_clock_speed, rhs_clock_speed

    if random.randint(0, 1):
        append(f, rhs, rhs_clock)
        append(f, lhs, lhs_clock)
        append(f, inter, rhs_clock)
        if (f.type == 0 and f.subtype == 8):
            sys.stdout.write("%d\t%d\n" % (len(lhs), len(rhs)))
    else:
        if (random.randint(0, 1)):
            append(f, lhs, lhs_clock)
        else:
            append(f, rhs, rhs_clock)
    append(f, union, rhs_clock)
    lhs_clock = lhs_clock + lhs_clock_speed * 100
    rhs_clock = rhs_clock + rhs_clock_speed * 100

if len(sys.argv) == 3:
    add_phy = 1
    Phy = globals()[sys.argv[2]]
    lt = { PrismHeader: 119,
           RadioTap:    127 }[Phy]
else:
    add_phy = 0
    lt = 105

sc = 42

for i in range(100):

    d11 = Dot11 (SC = sc << 4) / Dot11Beacon (timestamp = i * 100)

    if add_phy: gen(Phy () / d11)
    else:       gen(d11)

    sc = (sc + 1) % 4096

    for j in range(random.randint(0, 10)):

        d11 = Dot11 (type = 2, subtype = 4, SC = sc << 4)

        if add_phy: gen(Phy () / d11)
        else:       gen(d11)

        sc = (sc + 1) % 4096

wrpcap(sys.argv[1] + "-lhs.pcap",         lhs,    linktype = lt)
wrpcap(sys.argv[1] + "-rhs.pcap",         rhs,    linktype = lt)
wrpcap(sys.argv[1] + "-inter.pcap",       inter,  linktype = lt)
wrpcap(sys.argv[1] + "-union.pcap",       union,  linktype = lt)
